'use strict';

var Promise = require('bluebird');
var createDebug = require('debug');
var Utils = require('../utils');

module.exports = {
  run(tasks, runtime, flowOpts) {
    var debug = createDebug(`flow-wing{${flowOpts.name}}:runner:parallel`);
    var results = {};
    var errors = [];

    debug(`running ${tasks.length} tasks`);

    var promises = Promise.map(tasks, function (task, index) {
      var taskID = task.id || index;
      var passRuntime = task.flowAsTask;

      debug('initiating task %s', taskID);

      var taskPromise = task.run(runtime.context, passRuntime ? runtime : undefined).tap(function (result) {
        debug('task %s finished successfully', taskID);
        results[taskID] = result;
      }).catch(function (error) {
        debug('task %s has failed due to: %s', taskID, error.message);

        var taskError = Utils.buildTaskError(error, taskID, flowOpts);

        if (flowOpts.abortOnError) {
          throw taskError;
        } else {
          debug('continuing tasks execution because abortOnError=%s', flowOpts.abortOnError);
          results[taskID] = undefined;
          errors.push(taskError);
        }
      });

      // When flowOpts.abortOnError=false return the promise.reflect() so that
      // Promise.all(promises) below waits for all the promises to settle
      // even though one or more of them are rejected
      return flowOpts.abortOnError ? taskPromise : taskPromise.reflect();
    }, { concurrency: flowOpts.concurrency });

    return Promise.all(promises).then(function () {
      return { errors, results };
    });
  }
};