'use strict';

var Promise = require('bluebird');
var createDebug = require('debug');
var Utils = require('../utils');

/**
 * Waterfall and Series Tasks runner
 */
module.exports = {
  run(tasks, runtime, flowOpts) {
    var debug = createDebug(`flow-wing{${flowOpts.name}}:runner:${flowOpts.mode}`);
    var results = {};
    var errors = [];

    var skipPending = false;
    var pipeResult = runtime.opts.piped || runtime.opts.mode === 'waterfall';

    debug(`running ${tasks.length} tasks`);

    var promises = tasks.reduce(function (promise, task, index) {
      var taskID = task.id || index;
      var passRuntime = task.flowAsTask;

      return promise.then(function (value) {
        debug('initiating task %s', taskID);

        if (passRuntime) {
          runtime.previousResult = value;
          value = runtime;
        }

        if (!passRuntime && !pipeResult) {
          value = undefined;
        }

        return task.run(runtime.context, value);
      }).tap(function (result) {
        debug('task %s finished successfully', taskID);
        results[taskID] = result;
      }).catch(function (error) {
        if (skipPending) {
          throw error;
        }

        debug('task %s has failed due to: %s', taskID, error.message);

        skipPending = flowOpts.abortOnError;
        var taskError = Utils.buildTaskError(error, taskID, flowOpts);

        if (flowOpts.abortOnError) {
          throw taskError;
        } else {
          debug('continuing tasks execution because abortOnError=%s', flowOpts.abortOnError);
          results[taskID] = undefined;
          errors.push(taskError);
        }
      });
    }, Promise.resolve(runtime.previousResult));

    return promises.then(function () {
      return { errors, results };
    });
  }
};